@@ -1,13 +1,16 @@
 from typing import Any, Callable
 
+import numpy as np
 import pandas as pd
+import tensorflow as tf
 from sklearn.base import BaseEstimator, TransformerMixin
 from sklearn.pipeline import make_pipeline
 from sklearn.preprocessing import OrdinalEncoder
 
-from rexify.features.base import HasSchemaMixin, Serializable, TFDatasetGenerator
+from rexify.features.base import HasSchemaMixin, Serializable
 from rexify.features.transform import EventEncoder, Sequencer, TargetTransformer
 from rexify.schema import Schema
+from rexify.utils import get_target_id
 
 
 class _EventGenerator(BaseEstimator, TransformerMixin, HasSchemaMixin):
@@ -87,9 +90,72 @@ class _EventGenerator(BaseEstimator, TransformerMixin, HasSchemaMixin):
     def window_size(self):
         return self._window_size
 
+    @property
+    def ranking_features(self):
+        return self._ppl.steps[0][1].ranking_features
+
+    @property
+    def history(self):
+        return self._ppl.steps[1][1].history
+
+
+class _TFDatasetGenerator(HasSchemaMixin):
+    _event_gen: _EventGenerator
+
+    def make_dataset(self, X: pd.DataFrame) -> tf.data.Dataset:
+        ds = self._get_dataset(X)
+        ds = ds.map(self._get_header_fn())
+        return ds
+
+    def _get_dataset(self, data: pd.DataFrame) -> tf.data.Dataset:
+        return tf.data.Dataset.zip(
+            (
+                self._get_target_vector_dataset(data, self._schema, "user"),
+                self._get_target_vector_dataset(data, self._schema, "item"),
+                tf.data.Dataset.from_tensor_slices(
+                    np.stack(data["history"].values).astype(np.int32)
+                ),
+                self._get_ranking_dataset(data),
+            )
+        )
+
+    @staticmethod
+    def _get_target_vector_dataset(
+        data, schema: Schema, target: str
+    ) -> tf.data.Dataset:
+        return tf.data.Dataset.from_tensor_slices(
+            data.loc[:, get_target_id(schema, target)]
+            .values.reshape(-1)
+            .astype(np.int32)
+        )
+
+    def _get_ranking_dataset(self, data) -> tf.data.Dataset:
+        @tf.autograph.experimental.do_not_convert
+        def add_header(x):
+            return {
+                self._event_gen.ranking_features[i]: x[i]
+                for i in range(len(self._event_gen.ranking_features))
+            }
+
+        return tf.data.Dataset.from_tensor_slices(
+            data.loc[:, self._event_gen.ranking_features].values.astype(np.int32)
+        ).map(add_header)
+
+    @staticmethod
+    def _get_header_fn():
+        @tf.autograph.experimental.do_not_convert
+        def header_fn(user_id, item_id, history, ranks):
+            return {
+                "query": {"user_id": user_id, "history": history},
+                "candidate": {"item_id": item_id},
+                "rank": ranks,
+            }
+
+        return header_fn
+
 
 class FeatureExtractor(
-    BaseEstimator, TransformerMixin, Serializable, TFDatasetGenerator
+    BaseEstimator, TransformerMixin, Serializable, _TFDatasetGenerator
 ):
 
     _event_gen: _EventGenerator
@@ -98,33 +164,44 @@ class FeatureExtractor(
     def __init__(
         self,
         schema: Schema,
-        users_path: str = None,
-        items_path: str = None,
+        users: str = None,
+        items: str = None,
         load_fn: Callable = pd.read_csv,
+        return_dataset: bool = True,
+        window_size: int = 3,
     ):
         HasSchemaMixin.__init__(self, schema)
         self._user_transformer = TargetTransformer(schema, "user")
         self._item_transformer = TargetTransformer(schema, "item")
         self._load_fn = load_fn
 
-        self._users_path = users_path
-        self._items_path = items_path
+        self._users = users
+        self._items = items
+        self._return_dataset = return_dataset
+        self._window_size = window_size
 
     def fit(self, X, y=None):
         self._fit_extractor("user")
         self._fit_extractor("item")
         self._event_gen = _EventGenerator(
-            self._schema, self._user_transformer, self._item_transformer
+            schema=self._schema,
+            user_extractor=self._user_transformer,
+            item_extractor=self._item_transformer,
+            window_size=self._window_size,
         )
         self._event_gen.fit(X, y)
         self._model_params = self._get_model_params()
         return self
 
     def transform(self, X):
-        return self._event_gen.transform(X)
+        events = self._event_gen.transform(X)
+        self._model_params["session_history"] = self.history
+        if self._return_dataset:
+            return self.make_dataset(events)
+        return events
 
     def _fit_extractor(self, target: str):
-        path = getattr(self, f"_{target}s_path")
+        path = getattr(self, f"_{target}s")
         transformer = getattr(self, f"_{target}_transformer")
         data = self._load_fn(path)
         _ = transformer.fit(data).transform(data)
@@ -133,15 +210,17 @@ class FeatureExtractor(
         model_params = {}
         model_params.update(self._user_transformer.model_params)
         model_params.update(self._item_transformer.model_params)
+        model_params.update({"ranking_features": self.ranking_features})
+        model_params["window_size"] = self._window_size
         return model_params
 
     @property
-    def users_path(self):
-        return self._users_path
+    def users(self):
+        return self._users
 
     @property
-    def items_path(self):
-        return self._items_path
+    def items(self):
+        return self._items
 
     @property
     def load_fn(self):
@@ -150,3 +229,19 @@ class FeatureExtractor(
     @property
     def model_params(self):
         return self._model_params
+
+    @property
+    def ranking_features(self):
+        return self._event_gen.ranking_features
+
+    @property
+    def history(self):
+        return self._event_gen.history
+
+    @property
+    def return_dataset(self):
+        return self._return_dataset
+
+    @property
+    def window_size(self):
+        return self._window_size
